#!/bin/sh
# *******************************************************************************
#  Licensed Materials - Property of IBM
#  (c) Copyright IBM Corporation 2014. All Rights Reserved.
#  
#  Note to U.S. Government Users Restricted Rights:  
#  Use, duplication or disclosure restricted by GSA ADP Schedule 
#  Contract with IBM Corp. 
#  *******************************************************************************

# **************************************************************
# * Jazz application upgrade script for Jazz Reporting Service *
# **************************************************************


START_DIR=`pwd`
# Ensure the app_upgrade script was not called directly.
if [ "$SCRIPT_DIR" = "" ]; then
	INVALID_LAUNCH=true
	SCRIPT_DIR=`dirname $0`
else
	INVALID_LAUNCH=false
fi

# Read functions
. $SCRIPT_DIR/functions.sh
getAbsolutePath $SCRIPT_DIR
SCRIPT_DIR=$ABSOLUTE_PATH

# Messages script
. $SCRIPT_DIR/locale.sh
. $SCRIPT_DIR/messages/messages$LANG_SUFFIX.sh

# Exit in case of invalid launch
if [ $INVALID_LAUNCH = true ]; then
	messages INVALID_LAUNCH
	exit 10
fi

# First argument is the kind of upgrading
# Should be only -rs
if [ $# -eq 0 ]; then
	messages MISSING_APP_TYPE
	exit 10
fi
case $1 in
	-rs)
		UPGRADED_APP=`echo $1 | cut -c2-`
		shift
		;;
	*)
		messages INVALID_APP_TYPE $1
		exit 10
		;;
esac

# Set parameters values from file
. $SCRIPT_DIR/params/${UPGRADED_APP}_params$LANG_SUFFIX.sh

# Unset the default application context root to allow the user to set it
APPLICATION_CONTEXT_ROOT=

# Starting dir should be server dir
getAbsolutePath $SCRIPT_DIR/..
SERVER_DIR=`dirname $ABSOLUTE_PATH`
if [ $START_DIR != $SERVER_DIR ]; then
	if [ "$1" = "-help" ]; then
		help
	else
		messages INVALID_START_DIR $SCRIPT_NAME $SERVER_DIR
	fi
	exit 10
fi

# ***********************
# * Parse the arguments *
# ***********************
while [ $# -gt 0 ]
do
	case $1 in
		-oldApplicationHome | -oldapplicationhome )
			if [ $# -eq 1 ]; then
				messages MISSING_PARAM_VALUE $1
				exit 1
			fi
			if [ -d $2 ]; then
				if [ "`echo $2 | cut -d'/' -f1`" = "" ]; then
					OLD_APPLICATION_HOME=$2
					shift
				else
					messages RELATIVE_PARAM_DIR $1 $2
					exit 1
				fi
			else
				messages INVALID_PARAM_DIR $1 $2
				exit 1
			fi
			;;
		-applicationContextRoot | -applicationcontextroot)
			if [ $# -eq 1 ]; then
				messages MISSING_PARAM_VALUE $1
				exit 1;
			fi
			APPLICATION_CONTEXT_ROOT=$2
			shift
			;;
		-oldApplicationVersion | -oldapplicationversion)
			if [ $# -eq 1 ]; then
				messages MISSING_PARAM_VALUE $1
				exit 1
			fi
			OLD_APPLICATION_VERSION=$2
			shift
			;;
		-newApplicationHome | -newapplicationhome)
			if [ $# -eq 1 ]; then
				messages MISSING_PARAM_VALUE $1
				exit 1
			fi
			if [ -d $2 ]; then
				getAbsolutePath $2
				NEW_APPLICATION_HOME=$ABSOLUTE_PATH
				shift
			else
				messages INVALID_PARAM_DIR $1 $2
				exit 1
			fi
			;;
		-updateTomcatFiles | -updatetomcatfiles)
			if [ $# -eq 1 ]; then
				messages MISSING_PARAM_VALUE $1
				exit 1
			fi
			if [ "$2" != "yes" ] && [ "$2" != "no" ]; then
				messages INVALID_PARAM_VALUE $1 $2
				exit 1
			fi
			UPDATE_TOMCAT_FILES=$2
			shift
			;;
		-oldApplicationTomcatHome | -oldapplicationtomcathome)
			if [ $# -eq 1 ]; then
				messages MISSING_PARAM_VALUE $1
				exit 1
			fi
			if [ "`echo $2 | cut -d'/' -f1`" = "" ]; then
				OLD_APP_TOMCAT_HOME=$2
				shift
			else
				messages RELATIVE_PARAM_DIR $1 $2
				exit 1
			fi
			shift
			;;
		-newApplicationTomcatHome | -newapplicationtomcathome)
			if [ $# -eq 1 ]; then
				messages MISSING_PARAM_VALUE $1
				exit 1
			fi
			getAbsolutePath $2
			NEW_APP_TOMCAT_HOME=$ABSOLUTE_PATH
			shift
			;;
		-ignoreJTSVersionCheck | -ignorejtsversioncheck)
			IGNORE_JTS_VERSION_CHECK=ignoreJTSVersionCheck
			;;
		-newJTSHome | -newjtshome)
			if [ $# -eq 1 ]; then
				messages MISSING_PARAM_VALUE $1
				exit 1
			fi
			if [ -d $2 ]; then
				getAbsolutePath $2
				NEW_JTS_HOME=$ABSOLUTE_PATH
				shift
			else
				messages INVALID_PARAM_DIR $1 $2
				exit 1
			fi
			;;
		-jtsContextRoot | -jtscontextroot)
			if [ $# -eq 1 ]; then
				messages MISSING_PARAM_VALUE $1
				exit 1
			fi
			JTS_CONTEXT_ROOT=$2
			shift
			;;
		-noPrompt | -noprompt)
			NO_PROMPT=noPrompt
			;;
		-noVerify | -noverify)
			NO_VERIFY=false
			;;
		-noEditor)
			NO_EDITOR=true
			;;
		-overwrite)
			OVERWRITE=overwrite=yes
			;;		
		-noStepPrompt | -nostepprompt)
			NO_STEP_PROMPT=true
			;;
		-importReportsOnStartup | -importreportsonstartup)
			IMPORT_REPORTS_ON_STARTUP=importReportsOnStartup
			;;			
		-help)
			help
			exit 2
			;;
		*)
			messages INVALID_PARAM $1
			exit 1
			;;
	esac
	shift
done

# Determine the command file suffix depending on OS type
CMD_SUFFIX=sh
if [ "$OSTYPE" = "os400" ]; then
	#it ends with .qsh for v3.0 on IBM i	
		CMD_SUFFIX=qsh
fi

# *************************
# * Verify the parameters *
# *************************
#
#  -oldApplicationHome: 		mandatory, no default
#  -applicationContextRoot:		optional, default = application being updated (rs)
#  -oldApplicationVersion:		optional, no default
#  -newApplicationHome: 		mandatory, default = <scriptDirectory>/conf
#  -updateTomcatFiles:			optional, default = yes
#  -oldApplicationTomcatHome:	optional, default=tomcat
#  -newApplicationTomcatHome:	optional, default=tomcat
#
# For possible verification of the JTS version against the upgraded app version 
# the following parameters are needed
#  -ignoreJTSVersionCheck:	    optional, default is check app version against JTS version
#  -newJTSHome: 				optional, default = value of -newApplicationHome
#  -jtsContextRoot: 			optional, default = jts


# Determine the application context root and the previous application version
# ---------------------------------------------------------------------------
case $UPGRADED_APP in
	rs)
		REAL_CONTEXT_ROOT=`echo ${SCRIPT_NAME%_upgrade.sh}`
		if [ "$REAL_CONTEXT_ROOT" != "" ] && [ "$APPLICATION_CONTEXT_ROOT" = "" ]; then
			APPLICATION_CONTEXT_ROOT=$REAL_CONTEXT_ROOT

		fi
		if [ "$OLD_APPLICATION_VERSION" != "" ]; then
			OLD_APPLICATION_VERSION_ARG=oldApplicationVersion=$OLD_APPLICATION_VERSION
		fi
		;;
esac

# Verify that oldApplicationHome was specified.
# --------------------------------------------------------------------
if [ "$OLD_APPLICATION_HOME" = "" ]; then
	messages MISSING_HOME_PARAM -oldApplicationHome
	exit 3
fi


# Verify old application home has an app.properties file
# -------------------------------------------------------------
OLD_APP_TSP_FILE=$OLD_APPLICATION_HOME/$APPLICATION_CONTEXT_ROOT/app.properties
if [ ! \( -f $OLD_APP_TSP_FILE \) ]; then		
	messages TSP_NOT_FOUND_OLD_APP $OLD_APP_TSP_FILE $OLD_APPLICATION_HOME $APPLICATION_CONTEXT_ROOT		
	exit 3
fi


# Verify new application home has an app.properties file
# -------------------------------------------------------------
if [ "$NEW_APPLICATION_HOME" = "" ]; then
	messages MISSING_HOME_PARAM -newApplicationHome
	exit 3
fi
if [ ! \( -d $NEW_APPLICATION_HOME \) ]; then
	messages INVALID_PARAM_DIR -newApplicationHome $NEW_APPLICATION_HOME
	exit 3
fi
NEW_APP_TSP_FILE=$NEW_APPLICATION_HOME/$APPLICATION_CONTEXT_ROOT/app.properties
if [ ! \( -f $NEW_APP_TSP_FILE \) ]; then
	messages TSP_NOT_FOUND_NEW_APP $NEW_APP_TSP_FILE $NEW_APPLICATION_HOME $APPLICATION_CONTEXT_ROOT $OLD_APPLICATION_VERSION
	exit 3
fi


# Verify new application home has a com.ibm.team.integration.reporting.upgrade.jar
# ------------------------------------------------------------------------
cd $NEW_APPLICATION_HOME/..
NEW_APP_PARENT_DIR=`pwd`
UPGRADE_JAR="$NEW_APP_PARENT_DIR/upgrade/$APPLICATION_CONTEXT_ROOT/com.ibm.team.integration.reporting.upgrade.jar"
if [ ! \( -f $UPGRADE_JAR \) ]; then
	messages CMD_NOT_FOUND_NEW_APP $UPGRADE_JAR $NEW_APPLICATION_HOME $APPLICATION_CONTEXT_ROOT $OLD_APPLICATION_VERSION
	exit 3
fi
CLASSPATH=$UPGRADE_JAR:upgrade/$APPLICATION_CONTEXT_ROOT/*

if [ `uname` = Darwin ];
  then
    JAVA="/usr/bin/java"
  else
    JAVA="jre/bin/java"
fi
NEW_APPLICATION_CMD="$JAVA -classpath $CLASSPATH com.ibm.team.integration.reporting.upgrade.JRSTools"


# Verify Tomcat parameters: updateTomcatFiles, oldApplicationTomcatHome and newApplicationTomcatHome
# --------------------------------------------------------------------------------------------------
if [ "$UPDATE_TOMCAT_FILES" != "" ]; then
	UPDATE_TOMCAT_FILES_ARG="updateTomcatFiles=$UPDATE_TOMCAT_FILES"
fi

# Old Application Tomcat home
if [ "$OLD_APP_TOMCAT_HOME" != "" ]; then
	if [ ! \( -d $OLD_APP_TOMCAT_HOME \) ]; then
		if [ ! \( -d $OLD_APP_PARENT_DIR/$OLD_APP_TOMCAT_HOME \) ]; then
			if [ "`echo $OLD_APP_TOMCAT_HOME | cut -d'/' -f1`" = "" ]; then
				messages INVALID_PARAM_DIR -oldApplicationTomcatHome $OLD_APP_TOMCAT_HOME
			else
				messages INVALID_PARAM_DIR -oldApplicationTomcatHome $OLD_APP_PARENT_DIR/$OLD_APP_TOMCAT_HOME
			fi
			exit 3
		fi
	fi
	OLD_APP_TOMCAT_HOME_ARG="oldApplicationTomcatHome=$OLD_APP_TOMCAT_HOME"
fi

# New Application Tomcat home
if [ "$NEW_APP_TOMCAT_HOME" != "" ]; then
	if [ ! \( -d $NEW_APP_TOMCAT_HOME \) ]; then
		if [ ! \( -d $NEW_APP_PARENT_DIR/$NEW_APP_TOMCAT_HOME \) ]; then
			if [ "`echo $NEW_APP_TOMCAT_HOME | cut -d'/' -f1`" = "" ]; then
				messages INVALID_PARAM_DIR -newApplicationTomcatHome $NEW_APP_TOMCAT_HOME
			else
				messages INVALID_PARAM_DIR -newApplicationTomcatHome $NEW_APP_PARENT_DIR/$NEW_APP_TOMCAT_HOME
			fi
			exit 3
		fi
	fi
	NEW_APP_TOMCAT_HOME_ARG="newApplicationTomcatHome=$NEW_APP_TOMCAT_HOME"
fi


# Verify the parameters needed for JTS and app version checks
# -----------------------------------------------------------
if [ "$NEW_JTS_HOME" = "" ]; then
	NEW_JTS_HOME=$NEW_APPLICATION_HOME
fi
if [ ! \( -d $NEW_JTS_HOME \) ]; then
	messages INVALID_PARAM_DIR -newJTSHome $NEW_JTS_HOME
	exit 3
fi
if [ "$JTS_CONTEXT_ROOT" = "" ]; then
	JTS_CONTEXT_ROOT=jts
fi


# **************
# * Start exec *
# **************

messages BANNER_$UPGRADED_APP


# *********************
# * First step switch *
# *********************

cd $START_DIR

# Init variables
INITIAL_STEP=0

# Compute the maximum steps to execute
case $UPGRADED_APP in
	rs)		
		MAX_STEPS=1		
		;;
esac

# Get last execution information
LAST_STEP=-1
LAST_RETURN=0
LOG_FILE=$SCRIPT_DIR/.${UPGRADED_APP}_app_upgrade.log
if [ -f $LOG_FILE ]; then
	LAST_STEP=`cut -f1 -d' ' $LOG_FILE`
	LAST_RETURN=`cut -f2 -d' ' $LOG_FILE`
	LAST_STEP_INFO=`cut -f3 -d' ' $LOG_FILE`
	setFirstStep
else
	CURRENT_STEP=$INITIAL_STEP
fi

# **********************
# * Verify Public URIs *
# **********************
if [ $CURRENT_STEP -ne $INITIAL_STEP ]; then
	verify_all -clean
fi

# **************
# * Steps loop *
# **************
while [ $CURRENT_STEP -le $MAX_STEPS ]
do
	case $CURRENT_STEP in
		0)						
			CURRENT_STEP_MSG=STEP_0
			case $UPGRADED_APP in
				rs)
					CMD_ARGUMENTS="oldApplicationHome=$OLD_APPLICATION_HOME applicationContextRoot=$APPLICATION_CONTEXT_ROOT $OLD_APPLICATION_VERSION_ARG newApplicationHome=$NEW_APPLICATION_HOME  $UPDATE_TOMCAT_FILES_ARG $OLD_APP_TOMCAT_HOME_ARG  $NEW_APP_TOMCAT_HOME_ARG  $IGNORE_JTS_VERSION_CHECK newJTSHome=$NEW_JTS_HOME jtsContextRoot=$JTS_CONTEXT_ROOT $IMPORT_REPORTS_ON_STARTUP"
					STEP_CMD="$NEW_APPLICATION_CMD -migration_${UPGRADED_APP}_updateConfigurationFiles $CMD_ARGUMENTS"
					;;
			esac
			
			STEP_DIR=$NEW_APP_PARENT_DIR
			runStep $STEP_DIR
			
			# If step was not skipped
			# Verify app.properties and		
			if [ $? -eq 1 ]; then
				# Check error from merge, if it succeeded keep going
				# Fix for WI 193178
				if [ $ERROR_REPOTOOLS -eq 0 ]; then
					#skip this step for IBM i 
					if [ "$OSTYPE" != "os400" ] && [ "$NO_EDITOR" != "true" ]; then
						# Verify app.properties files
						messages EDIT_APP_PROPERTIES $UPGRADED_APP
						pause
						vi $NEW_APP_TSP_FILE
					fi
					# Verify all app.properties files
					verify_all
				fi	
				
				# Copy the application's database directory (server/conf/$app/db) only if it exists 
				# in the old application home and has not been copied yet to the new application home.
				copy_database $OLD_APPLICATION_HOME $APPLICATION_CONTEXT_ROOT $NEW_APPLICATION_HOME $APPLICATION_CONTEXT_ROOT
					
			fi
			;;
			
	esac

	# Store information to restart properly
	echo "$CURRENT_STEP $ERROR_REPOTOOLS" > $LOG_FILE
	
	# Verify that command was run successfully
	if [ $ERROR_REPOTOOLS -eq 0 ]; then
		if [ $CURRENT_STEP -eq $MAX_STEPS ]; then
			messages OVERALL_MIGRATION_SUCCESS_RS $UPGRADED_APP
		fi	
		CURRENT_STEP=`expr $CURRENT_STEP + 1`
	else
		messages OVERALL_MIGRATION_FAILED $UPGRADED_APP $CURRENT_STEP
		exit $ERROR_REPOTOOLS
	fi
done
