/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2014, 2015 All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function(){
dojo.provide("com.ibm.rdm.web.mergevvc.MergeStateStore");

// This is a store which maintains all the versioned resources taking place in the merge.
// The store keeps track of every resource by concept uri.  Those entries maintain which
// state - conflicted or resolved - each resource is currently in.  These versioned resources
// also know their conceptURI, so new versions as the result of a merge resolution can
// be tracked here.  Every resource can only be in one state or the other, but not both nor
// neither.  The status of a resource and conflict data, such as the total number of conflicts,
// is obtained via query.
//
// The store's data objects are in the form:
//		{
//			concept:			conceptURI,
//			resolveVersion:		The version of the artifact to be included in the merge result,
//			state:				One of the MergeState values, indicating whether the artifact is in conflict etc.
//			noMerge:			Boolean.  If present and true, indicates that the entry does not
//								represent an artifact that is to be directly involved in the merge, but it may have
//								related bookmarks links etc. which are to be merged. 
//		}

dojo.require("dojo.store.Memory");
dojo.require("dojo.store.util.QueryResults");
dojo.require("com.ibm.rdm.web.mergevvc.MergeState");
dojo.require("com.ibm.rdm.web.notifications.ChangeNotifier");
dojo.require("com.ibm.rdm.web.notifications.EventTopics");

var MergeState = com.ibm.rdm.web.mergevvc.MergeState,
	ChangeNotifier = com.ibm.rdm.web.notifications.ChangeNotifier,
	EventTopics = com.ibm.rdm.web.notifications.EventTopics;

dojo.declare("com.ibm.rdm.web.mergevvc.MergeStateStore", null, {
	
	// A MergeStateWriter to be used to write changes to merge state to VVC. 
	_writer: null,
	
	// A hash, keyed by concept URI, holding the merge state of a bunch of resources.
	_store: null,

	constructor: function(mergeStateWriter) {
		// mergeStateWriter:
		//		A writer through which changes to the version mappings in a merge can be written. 
		this._writer = mergeStateWriter;
		// Create the main hash keyed by conceptId
		this._store = {};
	},
	
	get: function(conceptURI) {	
		// summary:
		// 		Returns the store object for the given conceptURI
		// conceptURI:
		//		A string, which may or may not represent a conceptURI being stored
		//		in as the UID of one of our objects
		// returns:
		//		The object in the data store which the conceptURI is the unique key
		//		for.  Otherwise returns undefined.
	
		var item = this._store[conceptURI];
		return item;
	},
		
	put: function(concept, version, state, initialising) {
		// summary:
		//		Creates or updates an object in our store with attributes set to
		//		the initialization parameters.  If a new entry is created, or the
		//		state has changed on an existing object, a MERGE_STATE_CHANGE event
		//		is raised.
		// concept:
		//		ConceptURI string to be used as the UID for this object
		// version:
		//		Version string corresponding to a version of this resource
		// state:
		//		A string constant state as defined in MergeConstants
		// initialising:
		//		Optional boolean.  True to indicate that the store is being initialised with
		//		resources for which a merge version can be notified to VVC.
		//		When not initialising (i.e later when the states are being changed), any
		//		new entries put will be added to the store, but will not cause a version to
		//		be sent to VVC.  This allows addition of items that have related resources, links
		//		etc, but are not themselves changed. 
		// returns:
		//		A promise that resolves once the server has been notified of the new object version
		//		that is to be used in the merged configuration.  If there is an error, the underlying
		//		code will have displayed an error message before this promise is resolved.
		
		return this.putMulti([{
			concept: concept,
			resolveVersion: version,
			state: state
		}], initialising);
	},
		
	
	putMulti: function(entries, /*?boolean*/initialising) {
		// summary:
		//		Adds multiple entries to the store.  If notifications are to be sent
		//		[the default for .put()] a single message is sent to VVC, covering all
		//		the version changes.
		//		If a new entry is created, or the state has changed on an existing object,
		//		a MERGE_STATE_CHANGE notification event is raised.
		//		This method should be used in preference to plain .put() in situations
		//		where the versions on multiple resources are being updated.
		// entries:
		//		An array of entries of the form:
		//			{
		// 				concept:		ConceptURI string to be used as the UID for this object
		// 				resolveVersion:	Version string corresponding to a version of this resource.  May be undefined to indicate
		//								removal of the concept from the target.
		//				state:			A string constant state as defined in MergeConstants
		//			}
		// initialising:
		//		Optional boolean.  True to indicate that the store is being initialised with
		//		resources for which a merge version can be notified to VVC.
		//		When not initialising (i.e later when the states are being changed), any
		//		new entries put will be added to the store, but will not cause a version to
		//		be sent to VVC.  This allows addition of items that have related resources, links
		//		etc, but are not themselves changed. 
		// returns:
		//		A promise that resolves once the server has been notified of the new object versions
		//		that are to be used in the merged configuration.  If there is an error, the underlying
		//		code will have displayed an error message before this promise is resolved.
		
		var vvcNotifyEntries = [],
			mergeStateChangeEvents = [];
		dojo.forEach(entries, function(newEntryValues){
			var concept = newEntryValues.concept;
				entry = this._store[concept];
				if (!entry) {
					// create a new entry
					entry = this._store[concept] = {};
					if (!initialising) {
						// There was no corresponding entry added during initialisation.  This entry
						// is added with the noMerge flag to indicate that VVC should not be notified
						// but it may have related resources for which VVC does need to be notified of
						// version changes.
						entry.noMerge = true;
					}
					 
				}
			var previousState = entry.state,
				previousVersion = entry.resolveVersion,
				newState = newEntryValues.state,
				newVersion = newEntryValues.resolveVersion;
			// set the new values on the entry
			entry.concept = newEntryValues.concept;
			entry.resolveVersion = newEntryValues.resolveVersion;
			entry.state = newEntryValues.state;
		
			if (!initialising && ((previousState !== newState) || (previousVersion !== newVersion))) {
				// state has changed - add to list to tell VVC, unless the entry indicates that VVC should not be told.
				if (!entry.noMerge) {
					vvcNotifyEntries.push({
						concept: concept,
						version: newVersion
					});
				}
				
				// and notify any listeners so they can update their UI and update merge of related resources.
				mergeStateChangeEvents.push({
					eventTopic: EventTopics.MERGE_STATE_CHANGE,
					mergeStateEntry: entry
				});
			}
		}, this); // forEach
		
		// Bulk tell VVC of the new versions.  writeStateChanges is well-behaved if the list is empty.
		return this._writer.writeStateChanges(vvcNotifyEntries).then(function(results) {
			mergeStateChangeEvents.forEach(ChangeNotifier.notifyListeners, ChangeNotifier);
			return results;
		});
	}	
});
})();
