/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2015. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.rdm.web.query.filters.FilterByLinksOrAttributeValuesSection");

dojo.require("dijit._Templated");
dojo.require("dijit._Widget");
dojo.require("dijit._base");
dojo.require("dojo.DeferredList");

dojo.require("com.ibm.rdm.web.query.CompoundConditionChooser");
dojo.require("com.ibm.rdm.web.query.ConditionChooser");
dojo.require("com.ibm.rdm.web.query.conditionTypes");
dojo.require("com.ibm.rdm.web.query.filters.FilterGroupSection");


var CompoundConditionChooser = com.ibm.rdm.web.query.CompoundConditionChooser;
var ConditionChooser = com.ibm.rdm.web.query.ConditionChooser;
var conditionTypes = com.ibm.rdm.web.query.conditionTypes;


dojo.declare(
	"com.ibm.rdm.web.query.filters.FilterByLinksOrAttributeValuesSection",
	com.ibm.rdm.web.query.filters.FilterGroupSection,
{
	// summary:
	//		Provides creation and management functions for a group of
	//		ConditionChooser instances.
	// description:
	//		Instances of this class are created in the project dashboard's
	//		"Filter by Attribute" section to represent both "More attributes"
	//		and "Linked artifacts".  In addition other areas of the product,
	//		such as the artifact picker, instantiate this class to represent
	//		a "More attributes" filter section.

	///////////////////////////////////////////////////////////////////////////
	// Public fields

	attributesSection: null,

	// scopedArtifactTypeURIs: Array
	//		An array of the artifact type URIs that, when specified, should be
	//		used to narrow down the set of attributes that the user can select
	//		amongst.  For example if the user has:
	//		- custom attributes A, B, and C
	//		- attribute types MyType1 (using A, B and C), MyType2 (using B)
	//		  and MyType3 (using B and C)
	//		and the user does the following:
	//		- selects attribute types MyType2 and MyType3
	//		- clicks on "More attributes" (to define a new attribute condition)
	//		the resulting attribute condition chooser should ONLY allow
	//		selecting amoung attributes B and C (and not A as neither MyType2
	//		nor MyType3 contains attribute A).
	//		This field should ONLY be passed if the viewType is set to
	//		AttributeValue (as it doesn't make sense if the viewType is
	//		LinkedArtifact).
	scopedArtifactTypeURIs: null,

	// blacklistAttributeDefinitionURIs: Array
	//		An array of attribute URIs that should not be shown in this filter.
	blacklistAttributeDefinitionURIs: null,

	// restrictedValuesToUseInArtifactFormats: [Object] {[Array] artifactFormats, [Array] mimeTypes}
	//		By default, the "Artifact format" filter shows with artifact formats
	//		followed by all registerd MIME types.  If this field is passed, the filter will instead 
	//		show the artifact formats and/or the mime types that are specified.  If no value is specified for
	//		one of the filters, all values will be displayed for that filter. Example values: 
	//		
	//		Only jpg type files will be displayed, no artifact formats will be displayed.
	//		  {
	//		    artifactFormats: [],
	//			mimeTypes: [com.ibm.rdm.web.common.MimeTypes.JPG]
	//		  }
	//		
	//		jpg type files and all artifact formats will be displayed.
	//		  {
	//			mimeTypes: [com.ibm.rdm.web.common.MimeTypes.JPG]
	//		  }
	restrictedValuesToUseInArtifactFormats: null,

	///////////////////////////////////////////////////////////////////////////
	// Private fields

	_conditionChoosers: null,
	_isInProcessOfCallingSetConditionInfos: null,
	_isRefreshInProgress: null,
	_initialPlacementOfApplyHasOccurred: null,

	///////////////////////////////////////////////////////////////////////////
	// Public API

	postCreate: function() {
		this.inherited(arguments);
		if (!this.blacklistAttributeDefinitionURIs) {
			this.blacklistAttributeDefinitionURIs = [];
		}
		this._conditionChoosers = [];
		// Note: This class is overruling the onclick behavior for the
		// superclass FilterGroupSection ... rather than expand/collapse
		// we're wiring it to end up invoking this._onTitleAreaClick.
		// This is why we're setting the _contentSectionDiv display style to ""
		// in the following ...
		this._contentSectionDiv.style.display = "";
		this._hideApplySection();
	},

	postMixInProperties: function() {
		this.inherited(arguments);
		if (! this.sectionTitle) {
			switch (this.viewType) {
			case com.ibm.rdm.web.query.filters.FilterByLinksOrAttributeValuesSection.viewType.AttributeValue:
				this.sectionTitle = this._messages["filterByAttribute"];
				break;
			case com.ibm.rdm.web.query.filters.FilterByLinksOrAttributeValuesSection.viewType.LinkedArtifact:
				this.sectionTitle = this._messages["filterByLinkedArtifact"];
				break;
			}
		}
	},

	destroy: function() {
		this.inherited(arguments);
		dojo.forEach(
			this._conditionChoosers,
			function(conditionChooser) {
				conditionChooser.destroyRecursive();
			}
		);
	},

	startup: function() {
		if (!this._started) {
			dojo.forEach(
				this._conditionChoosers,
				function(conditionChooser) {
					conditionChooser.startup();
				}
			);
		}
		this.inherited(arguments);
	},

	refresh: function(onRefreshComplete) {
		// summary:
		//		This is an override of a superclass function.
		var self = this;
		this._isRefreshInProgress = true;
		var performOnRefreshComplete = function() {
			self._isRefreshInProgress = false;
			onRefreshComplete();
		};
		if (this._conditionChoosers.length > 0) {
			var numOutstandingDeferreds = this._conditionChoosers.length;
			var chooserIndicesToRemove = [];
			var onConditionChooserRefreshCompleted = function(
				conditionChooser,
				conditionChooserIndex
				) {
				// Note: After obtaining "fresh" attribute definition/type info
				// it MAY be the case that the current filter is no longer valid.
				// For example, if the filter is based on an enum field value,
				// perhaps this field value has been removed.  If the filter is
				// no longer valid we take care here to remove the filter BEFORE
				// telling the the consumer of this class (via
				// onRefreshComplete) that we're up to date (as we want
				// to make sure that this invalid filter DOES NOT get added to
				// the query that is subsequently issued).
				if (conditionChooser.isAttributeConditionSpecifiedButNoLongerValid()) {
					self.clearFilter();
				}
				if (conditionChooser.isCollapsed()
					&& conditionChooser.isInvalid()
					) {
					// This chooser represents one a chooser in the "More
					// attributes" section that's associated with an attribute
					// that's been deleted.
					chooserIndicesToRemove.push(conditionChooserIndex);
				}
				numOutstandingDeferreds--;
				if (numOutstandingDeferreds === 0) {
					if (chooserIndicesToRemove.length > 0) {
						// We take care here to remove the condition choosers
						// in such a manner that removing one would not screw
						// up the index of each of the subsequent stored
						// indices.
						var indicesSortedHighestToLowest =
							chooserIndicesToRemove.sort(function(a, b){ return b-a; });
						dojo.forEach(
							indicesSortedHighestToLowest,
							function(conditionChooserIndex) {
								self.removeAppliedAttributeValue(conditionChooserIndex);
							}
						);
					}
					performOnRefreshComplete();
				}
			};
			var refreshConditionChooser = function(conditionChooser, conditionChooserIndex) {
				conditionChooser.refresh(
					function() {
						onConditionChooserRefreshCompleted(
							conditionChooser, conditionChooserIndex
						);
					}
				);
			};
			for (var i = 0; i < this._conditionChoosers.length; i++) {
				refreshConditionChooser(this._conditionChoosers[i], i);
			}
		} else {
			performOnRefreshComplete();
		}
	},

	isFilterApplied: function() {
		// summary:
		//		This is an override of a superclass function.
		// description:
		//		Rather than muck with attempting to set _isFilterApplied
		//		when needed, we're instead taking the approach of
		//		overriding this function ...
		var isAtLeastOneChooserCollapsed = false;
		if (this._conditionChoosers.length > 1) {
			isAtLeastOneChooserCollapsed = true;
		}
		else if (this._conditionChoosers.length === 1) {
			var conditionChooser = this._conditionChoosers[0];
			isAtLeastOneChooserCollapsed = (
				conditionChooser instanceof com.ibm.rdm.web.query.ConditionChooser ?
				conditionChooser.isCollapsed() :
				// It's a CompoundConditionChooser ...
				conditionChooser.isAtLeastOneChooserCollapsed()
			);
		}
		return isAtLeastOneChooserCollapsed;
	},

	isFilterSpecified: function() {
		// summary:
		//		This is an override of a superclass function.
		var isFilterSpecified = false;
		if (this._conditionChoosers.length > 1) {
			isFilterSpecified = true;
		}
		else if (this._conditionChoosers.length === 1) {
			isFilterSpecified =
				this._conditionChoosers[0].isConditionStatementSpecified();
		}
		return isFilterSpecified;
	},

	clearFilter: function(skipApply) {
		// summary:
		//		This is an override of a superclass function.
		if (this._conditionChoosers.length > 0) {
			for (var i = 0; i < this._conditionChoosers.length; i++) {
				this._conditionChoosers[i].destroyRecursive();
			}
			this._conditionChoosers = [];
		}
		this._hideApplySection();
	},

	removeAppliedAttributeValue: function(conditionChooserIndex) {
		var conditionChooser =
			this._conditionChoosers[conditionChooserIndex];
		this._conditionChoosers.splice(conditionChooserIndex, 1);
		conditionChooser.destroyRecursive();
	},

	// TODO consider renaming this to be getAppliedConditionInfos
	getAppliedAttributeValuesInfo: function() {
		// summary:
		//		Returns an array of property lists, each property list is
		//		of a format described by the getConditionInfo function
		//		in ConditionChooser.
		var conditionInfos = [];
		dojo.forEach(
			this._conditionChoosers,
			function(conditionChooser) {
				if (conditionChooser.isConditionStatementSpecified()
					&& (conditionChooser instanceof com.ibm.rdm.web.query.ConditionChooser ?
						conditionChooser.isCollapsed() :
						// Note: It must be the case that at least one ConditionChooser
						// within our CompoundConditionChooser instance is collapsed
						// (as that's a requirement for the above call to
						// isConditionStatementSpecified for a CompoundConditionChooser).
						true)
					) {
					var conditionInfo =
						conditionChooser.getConditionInfo();
					conditionInfos.push(conditionInfo);
				}
			}
		);
		return conditionInfos;
	},

	// TODO Consider renaming this to be setAppliedConditionInfos.
	setAppliedAttributeValuesInfo: function(conditionInfos) {
		// summary:
		//		Initialize this filter field by applying the information
		//		found within the elements represented by conditionInfos.
		// conditionInfos:
		//		An array of property lists, each property list is
		//		of a format described by the getConditionInfo function
		//		in ConditionChooser.
		
		var deferredArray = [];
		if (conditionInfos.length > 0) {
			this._isInProcessOfCallingSetConditionInfos = true;
		}
			
		dojo.forEach(
			conditionInfos,
			function(conditionInfo) {
				var conditionChooser = this._addConditionChooser();
				deferredArray.push(conditionChooser.setConditionInfo(
					conditionInfo));
			},
			this
		);
		
		var deferredList = new dojo.DeferredList(deferredArray);
		
		if (deferredArray && deferredArray.length > 0) {
			deferredList.addCallback(this, function(){
				for (var conditionChooser in this._conditionChoosers){
					this._conditionChoosers[conditionChooser].expand(true);
					this._conditionChoosers[conditionChooser].collapse();
				}
				this._contentAreaHeightHasChanged();
				this._isInProcessOfCallingSetConditionInfos = false;
			});
		}

		return deferredList;
	},
	
	onFocus: function() {
		if (this._isFocusInHeaderArea()) {
			this._showAsMouseOver();
			this._showAsFocused();
		} else {
			this._stopShowingAsMouseOver();
			this._stopShowingAsFocused();
		}
		this._updateTitleAreaDisplay();
	},

	///////////////////////////////////////////////////////////////////////////
	// Private API

	_getExpandedConditionChooser: function() {
		var expandedConditionChooser = null;
		for (var i = this._conditionChoosers.length-1; i >= 0; i--) {
			var conditionChooser = this._conditionChoosers[i];
			if (conditionChooser.isExpanded()) {
				expandedConditionChooser = conditionChooser;
				break;
			}
		}
		return expandedConditionChooser;
	},

	_setScopedArtifactTypeURIsAttr: function(scopedArtifactTypeURIs) {
		this.scopedArtifactTypeURIs = scopedArtifactTypeURIs;
		var expandedConditionChooser =
			this._getExpandedConditionChooser();
		if (expandedConditionChooser) {
			// Tell this chooser what artifact types are now in effect.  We make
			// this call so that the list of selectable attributes within the
			// chooser will be narrowed down to just those that pertain to these
			// artifact types.
			expandedConditionChooser.attr(
				"scopedArtifactTypeURIs", scopedArtifactTypeURIs
			);
		}
	},

	_onKeyDown: function(event) {
		// summary:
		//		This is an override of a superclass function.
		if (event.target === this._sectionTitleDiv
			&& (event.keyCode === dojo.keys.SPACE || event.keyCode === dojo.keys.ENTER)
			) {
			dojo.stopEvent(event);
			// Note: If we were to call toggleSectionExpansionState directly
			// here it would result in the key release being consumed
			// by the first drop down button in the first ConditionChooser of a
			// CompoundConditionChooser, which results in opening this drop
			// down.  To avoid this, we're using the workaround of using a
			// settimeout here ...
			window.setTimeout(
				dojo.hitch(this, this.toggleSectionExpansionState),
				0
			);
		}
		return event;
	},

	_createContent: function() {
		// summary:
		//		This is an override of a superclass function.
		// TODO - need to deal with Apply button not being used
		this._addConditionChooser();
	},

	_getSummaryInfo: function() {
		// summary:
		//		This is an override of a superclass function.
		var label = "";
		return {
			type: "single-label",
			label: label
		};
	},

	_filterHasBeenApplied: function() {
		// summary:
		//		This is an override of a superclass function.
	},

	_filterHasBeenRemoved: function() {
		// summary:
		//		This is an override of a superclass function.
	},

	_focus: function() {
		// summary:
		//		This is an override of a superclass function.
		var expandedConditionChooser =
			this._getExpandedConditionChooser();
		if (expandedConditionChooser) {
			expandedConditionChooser.focus();
		}
	},

	_isMultipleFiltersSupported: function() {
		// summary:
		//		This is an override of a superclass function.
		return true;
	},

	_isSubFilterExpanded: function() {
		// summary:
		//		This is an override of a superclass function.
		return (this._getExpandedConditionChooser() !== null);
	},

	_invokeApplyAction: function(stopDashboardRefresh) {
		// summary:
		//		This is an override of a superclass function.
		var expandedConditionChooser = this._getExpandedConditionChooser();
		if (expandedConditionChooser) {
			expandedConditionChooser.collapse();
		}
		if (!stopDashboardRefresh) {
			this._selectionHasChanged();
		}
		this._hideApplySection();
		if (expandedConditionChooser) {
			expandedConditionChooser.focus();
		}
	},

	_contentAreaHeightHasChanged: function(/*Widget?*/chooser) {
		// summary:
		//		This is an override of a superclass function.
		if (this._started
			&& !this._isInProcessOfCallingSetConditionInfos
			) {
			if (chooser) {
				if (!this._isApplySectionShowing()
					&& chooser.domNode.parentNode
					&& chooser.domNode.parentNode.parentNode
					&& !this._isRefreshInProgress
					) {
					this._showApplySection(chooser);
				}
			}
			else if (!this._initialPlacementOfApplyHasOccurred
					 && this._conditionChoosers.length === 1) {
				this._initialPlacementOfApplyHasOccurred = true;
				this._showApplySection(this._conditionChoosers[0]);
			}
		}
		this.inherited(arguments);
	},

	_onTitleAreaClick: function(/*DOM event?*/event) {
		// summary:
		//		This is an override of a superclass function.
		if (event) {
			dojo.stopEvent(event);
		}
		if (!this._started) {
			this.startup();
		}
		if (!this._isSubFilterExpanded()) {
			this._stopShowingAsFocused();
			this._stopShowingAsMouseOver();
			this._onCreateClicked();
			this._updateTitleAreaDisplay();
		}
	},

	_onCreateClicked: function(/*DOM event?*/event) {
		// summary:
		//		This is an override of a superclass function.
		if (event) {
			dojo.stopEvent(event);
		}
		this._disableApplySection();
		this._addConditionChooser();
		this.inherited(arguments);
	},

	_getCurrentlySpecifiedAttributeDefinitionURIs: function() {
		var currentlySpecifiedAttributeDefinitionURIs = [];
		dojo.forEach(
			this._conditionChoosers,
			function(conditionChooser) {
				if (conditionChooser.isCollapsed()) {
					currentlySpecifiedAttributeDefinitionURIs.push(
						conditionChooser.getAttributeDefinitionUri());
				}
			}
		);
		return currentlySpecifiedAttributeDefinitionURIs;
	},

	_getAttributeDefinitionURIsThatShouldNotBeShown: function() {
		return this._getCurrentlySpecifiedAttributeDefinitionURIs().concat(
					this.blacklistAttributeDefinitionURIs);
	},

	_isApplySectionShowing: function() {
		return this._applySectionDiv.style.display !== "none";
	},

	_hideApplySection: function() {
		this._applySectionDiv.style.display = "none";
		this._contentAreaHeightHasChanged();
	},

	_showApplySection: function(chooser) {
		dojo.place(this._applySectionDiv, chooser.domNode, "after");
		this._applySectionDiv.style.display = "";
		this._contentAreaHeightHasChanged(chooser);
	},

	_enableApplySection: function(chooser) {
		// summary:
		//		This is an override of a superclass function.
		// chooser:
		//		An instance of a ConditionChooser or a CompoundConditionChooser.
		this.inherited(arguments);
	},

	_disableApplySection: function(chooser) {
		// summary:
		//		This is an override of a superclass function.
		// chooser:
		//		An instance of a ConditionChooser or a CompoundConditionChooser.
		this.inherited(arguments);
	},

	_addConditionChooser: function() {
		var conditionChooser;
		switch (this.viewType) {
		case com.ibm.rdm.web.query.filters.FilterByLinksOrAttributeValuesSection.viewType.AttributeValue:
			conditionChooser = new ConditionChooser({
				projectURI: this.attributesSection.projectURI,
				baselineURI: this.attributesSection.baselineURI,
				conditionType: conditionTypes.AttributeCondition,
				scopedArtifactTypeURIs: this.scopedArtifactTypeURIs,
				previouslySelectedAttributeDefinitionURIs: this._getAttributeDefinitionURIsThatShouldNotBeShown(),
				restrictedValuesToUseInArtifactFormats: this.restrictedValuesToUseInArtifactFormats,
				onRemoveFilter: dojo.hitch(this, this._removeConditionChooser),
				onEditFilter: dojo.hitch(this, this._editConditionChooser),
				onInvokeApplyAction: dojo.hitch(this, this._invokeApplyAction),
				onEnableApplySection: dojo.hitch(this, this._enableApplySection),
				onDisableApplySection: dojo.hitch(this, this._disableApplySection),
				onContentAreaHeightHasChanged: dojo.hitch(this, this._contentAreaHeightHasChanged)
			});
			break;
		case com.ibm.rdm.web.query.filters.FilterByLinksOrAttributeValuesSection.viewType.LinkedArtifact:
			conditionChooser = new CompoundConditionChooser({
				projectURI: this.attributesSection.projectURI,
				baselineURI: this.attributesSection.baselineURI,
				conditionType: conditionTypes.LinkTypeCondition,
				fixedAttributeName: this._messages["linkType"],
				onRemoveFilter: dojo.hitch(this, this._removeConditionChooser),
				onEditFilter: dojo.hitch(this, this._editConditionChooser),
				onInvokeApplyAction: dojo.hitch(this, this._invokeApplyAction),
				onEnableApplySection: dojo.hitch(this, this._enableApplySection),
				onDisableApplySection: dojo.hitch(this, this._disableApplySection),
				onContentAreaHeightHasChanged: dojo.hitch(this, this._contentAreaHeightHasChanged),
				filterCLMLinks: true
			});
			break;
		}
		this._contentDiv.appendChild(conditionChooser.domNode);
		conditionChooser.startup();
		if (!this._isInProcessOfCallingSetConditionInfos) {
			conditionChooser.focus();
		}
		this._conditionChoosers.push(conditionChooser);
		this._contentAreaHeightHasChanged(conditionChooser);
		this._listenForChildFocusAndBlurEvents(conditionChooser);
		return conditionChooser;
	},

	_removeConditionChooser: function(conditionChooser, skipApply) {
		// Remove the passed attribute condition chooser.
		var chooserIndex = dojo.indexOf(this._conditionChoosers, conditionChooser);
		conditionChooser.destroyRecursive();
		this._conditionChoosers.splice(chooserIndex, 1);
		if (!skipApply) {
			this._selectionHasChanged();
		}
		var expandedConditionChooser =
			this._getExpandedConditionChooser();
		if (expandedConditionChooser) {
			// Update the "black list" attributes as one more should now
			// appear within the chooser (i.e the one for which a filter
			// has just been removed).
			expandedConditionChooser.attr(
				"previouslySelectedAttributeDefinitionURIs",
				this._getCurrentlySpecifiedAttributeDefinitionURIs()
			);
		} else {
			this._hideApplySection();
			this.focus();
		}
	},

	_listenForChildFocusAndBlurEvents: function(conditionChooser) {
		this.connect(conditionChooser, "onFocus", function(){
			// When the user uses Tab to leave the header area and go into the
			// ConditionChooser we need to remove the onmouseover treatment
			// (i.e. orange color and display of a toolbar).
			this._stopShowingAsMouseOver();
			this._stopShowingAsFocused();
			this._updateTitleAreaDisplay();
		});
		this.connect(conditionChooser, "onBlur", function(){
			// If the user uses ShiftTab to leave the condition chooser, it
			// MAY be the case that the header area should now show using the
			// onmouseover treatment (i.e. orange color and display of a
			// toolbar).
			if (this._contentDiv.firstChild === conditionChooser.domNode
				&& this._isFocusInHeaderArea()
				) {
				this._showAsMouseOver();
				this._showAsFocused();
				this._updateTitleAreaDisplay();
			}
		});
	},

	_editConditionChooser: function(conditionChooser) {
		if (!this._isApplySectionShowing()) {
			this._showApplySection(conditionChooser);
		}
		for (var i = 0; i < this._conditionChoosers.length; i++) {
			if (conditionChooser !== this._conditionChoosers[i]) {
				var chooser = this._conditionChoosers[i];
				if (chooser.isExpanded()) {
					if (chooser.isConditionStatementSpecified()) {
						chooser.collapse();
					} else {
						this._removeConditionChooser(chooser, true);
					}
				}
			}
		}
		this._selectionHasChanged();
	}
});


})();


com.ibm.rdm.web.query.filters.FilterByLinksOrAttributeValuesSection.viewType = {
	AttributeValue: "attributeValue",
	LinkedArtifact: "linkedArtifact"
};
