/**********************************************************************************************************************************
 * Licensed Materials - Property of IBM
 * Restricted Materials of IBM.  Reproduction, modification and redistribution are prohibited.
 * Task-Oriented Quick Tracking and Planning feature for Rational Team Concert and Jazzhub.
 * (c) Copyright IBM Corporation 2014.
 * U.S. Government Users Restricted Rights:  Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp. 
 **********************************************************************************************************************************/

/**
 * Provides a Backlog editor.
 */
define(['framework/App', 'backbone', 'marionette', 'underscore',
		'framework/common/models/Action',
        'areas/timelines/models/TimelineCollection',
        'areas/iterations/models/IterationCollection',
        'areas/iterations/models/IterationConstants',
        'areas/workitems/models/WorkItemConstants',
        "areas/projects/models/ProjectConstants",
        'areas/workitems/models/SprintPlanTreeSection', 'areas/workitems/models/SectionCollection',
        'areas/workitems/controllers/AbstractWorkItemCollectionController',
        'areas/workitems/controllers/AbstractPlanningPageController',
    	'areas/workitems/controllers/utils/TreeViewParentChildModelUpdater',
        'areas/workitems/views/BacklogGroupCollectionView',
		'areas/filters/models/FilterConstants'], 
function(App, Backbone, Marionette, _,
		Action,
		TimelineCollection, 
		IterationCollection,
		IterationConstants, 
		Constants, 
		ProjectConstants,
		SprintPlanTreeSection, 
		SectionCollection,
		AbstractWorkItemCollectionController, 
		AbstractPlanningPageController,
		TreeViewParentChildModelUpdater,
		BacklogGroupCollectionView,
		FilterConstants) {
	
	var BacklogPageController= AbstractPlanningPageController.extend({

		_parentChildModelUpdater: new TreeViewParentChildModelUpdater(),
		
		// -- Move following to constructor ---
		
		canDisplayAsTable: function() {
			return true;
		},
		
		canDisplayAsList: function() {
			return false;
		},
		
		canDisplayAsLanes: function() {
			return false;
		},
		
		// --------   Over-riding parent methods -----------------
		
		createOptions: function(hints, viewType) {
			var options = AbstractWorkItemCollectionController.prototype.createOptions.call(this, _.extend(hints, {
				doNotTrimReleaseIterationForJazzhub: true
			}), viewType);
			
			// leave blank so that the first section gets selected by default
			options.defaultSectionId = null;
			
			options.title = App.messages.BacklogController_backlogLabel;
			
			// this is needed for setting URL location
			options.context.planType = Constants.backlog;
			options.showGroupItemCount = false;
			options.showBacklogGroupItemCount = true;
			options.GroupCollectionViewClass = BacklogGroupCollectionView;
			
			return options;
		},
		
		createSections: function(hints, viewType) {
			var context = hints.context;
			var section= this._createBacklogSection(hints);
			if (section) {
    			section.set('createOptions', {
    			    excludeFromNavbar: true
    			});
            }
			return new SectionCollection(section ? [section] : []);
		},
		
		/**
		 * Given an item and the section's functional ID, supplies the group name
		 * under which the item should fall.
		 */
		getGroup: function(item, functionalId) {
			return App.messages.BacklogController_rankedList;
		},
		
		getEmptyViewOptions: function(context) {
			return {'message': App.messages.BacklogController_emptyRankList };
		},
		
		getEditActions: function() {
			var editActions = {};
			editActions[Constants.propPriority] = true;
			editActions[Constants.propTags] = true;
			editActions[Constants.propOwnedBy] = true;
			editActions[Constants.propStatus] = true;
			editActions[Constants.propIsStarred] = true;
			editActions[Constants.editRefresh] = true;
			editActions[Constants.propChildren] = true;
			return editActions;
		},
		
		getTableViewHeaders: function() {
			var expandCollapseHeader = {name:"expand",label:"",size:"30px",align:"center"};
			var iconHeader = {name:"icon",label:"",size:"25px",align:"center"};
			var summaryHeader = {name:Constants.propSummary,label:"&nbsp;", align:"left"};
			var rankHeader = {name:Constants.propRank,label: App.messages.BacklogPageController_rank,size:"40px", align:"center"};
			var asArray = [expandCollapseHeader, iconHeader, summaryHeader, rankHeader];
			if (this.complexityAttributeID) {
				var complexityHeader= {name:Constants.propWiComplexityID, label: this.complexityAttributeLabel, size:"90px", align:"center"};
				asArray.push(complexityHeader);
			}
			var ownerHeader = {name:Constants.propOwnedBy,label: App.messages.BacklogPageController_owner, size:"135px", align:"left"};
			asArray.push(ownerHeader);
			var statusHeader = {name:Constants.propStatus,label: App.messages.BacklogPageController_status, size:"120px", align:"left"};
			asArray.push(statusHeader);
			var priorityHeader = {name:Constants.propPriority,label: App.messages.BacklogPageController_priority, size:"120px", align:"left"};
			asArray.push(priorityHeader);
			var actionsHeader = {name:"actions",label: App.messages.BacklogPageController_actions, size:"80px", align:"center"};
			asArray.push(actionsHeader);
			
			return this.formatTableViewHeaders(asArray);
		},
		
		// ----------------- Implementation --------------------
		
		_createBacklogSection: function(hints) {
			var newContext = _.clone(hints.context);
			var iteration= App.controller.iterationController.getBacklogIteration(newContext.project);
			if (!iteration) {
				return null;
			}
			newContext.iteration = iteration;
			
			var initialExcludeResolved = this.options.isBacklogOnlyView ? 
					true : App.preferences.at(0).get(FilterConstants.excludeResolvedPreferencesKey);
			
			return new SprintPlanTreeSection({
				context: newContext,
				initialExcludeResolved: initialExcludeResolved,
				getGroup: this.createGroupingFunction(iteration.get('id')),
				columns: this.getColumnIdentifiers(),
				sortAscending: this.getSortAscendingColumns(),
				sortDescending: this.getSortDescendingColumns(),
				workItemViewOptions: {
					tableHeaders: this.getTableViewHeaders(),
					editActions: this.getEditActions(),
					emptyViewOptions: this.getEmptyViewOptions(newContext),
					isTree: true,
					viewType: hints.viewType
				}
			});
		}, 
		
		refreshAndValidateItem: function(workItems, attributes) {
			var updateNeeded = _.any(workItems, _.bind(function(workItem, index) { return this._parentChildModelUpdater.updateParentChildModel(workItem, attributes[index]); }, this));
			return updateNeeded ?
						AbstractWorkItemCollectionController.prototype.refreshAndValidateItem.apply(this, arguments) :
						$.Deferred().resolve(workItems);
		}
	});
	
	return BacklogPageController;
});
